==> m3-ui-2.2.108.tar.gz: m3-ui-2.2.108/src/m3_ext/ui/js_template_loader.py <==
# coding: utf-8
"""
Загрузчик шаблонов (темплейтов) для генерации пользовательского интерфейса
для m3_ext_demo.ui.

Необходимость данного шаблона обусловлена спецификой реализации
template-loaders в django.
"""
import os
from os.path import (
    dirname,
    join as join_path,
)
import six

from django.conf import (
    settings,
)
from django.core.exceptions import (
    SuspiciousOperation,
)
from django.template import (
    Origin,
    TemplateDoesNotExist,
)
from django.utils._os import (
    safe_join,
)

from m3_django_compat import (
    BaseLoader,
)


# At compile time, cache the directories to search.
module_dir = dirname(__file__)

if six.PY2:
    # Пути должны быть закодированы в UTF-8
    # Во втором питоне module_dir содержит str в системной кодировке,
    # поэтому ее надо правильно декодировать
    import sys

    fs_encoding = sys.getfilesystemencoding() or sys.getdefaultencoding()
    template_dir_ext = join_path(module_dir, 'templates')
    template_dir_gears = join_path(dirname(module_dir), 'gears', 'templates')
    app_template_dirs = (
        template_dir_ext.decode(fs_encoding),
        template_dir_gears.decode(fs_encoding),
    )
else:
    # В третьем питоне str сразу содержит текст в нужной кодировке
    app_template_dirs = (
        join_path(module_dir, 'templates'),
        join_path(dirname(module_dir), 'gears', 'templates'),
    )


def get_template_sources(template_name, template_dirs=None):
    """
    Returns the absolute paths to "template_name", when appended to each
    directory in "template_dirs". Any paths that don't lie inside one of the
    template dirs are excluded from the result set, for security reasons.
    """
    if not template_dirs:
        template_dirs = app_template_dirs
    for template_dir in template_dirs:
        try:
            yield safe_join(template_dir, template_name)
        except UnicodeDecodeError:
            # The template dir name was a bytestring that wasn't valid UTF-8.
            raise
        # В Django <1.8.X safe_join выбрасывает ValueError
        # В Django >=1.8.X safe_join выбрасывает SuspiciousOperation
        except (SuspiciousOperation, ValueError):
            # The joined path was located outside of template_dir.
            pass


class Loader(BaseLoader):

    is_usable = True

    # TODO удалить после окончания поддержки Django < 2.0
    def load_template_source(self, template_name, template_dirs=None):
        return load_template_source(template_name, template_dirs)

    def get_template_sources(self, template_name):
        """
        Возвращает объект Origin для каждого найденного шаблона.
        """
        origins = []
        template_paths = get_template_sources(template_name)

        for template_path in template_paths:
            if os.path.isfile(template_path):
                origins.append(
                    Origin(
                        name=template_path,
                        template_name=template_name,
                        loader=self,
                    )
                )

        return origins


def load_template_source(template_name, template_dirs=None):
    for filepath in get_template_sources(template_name, template_dirs):
        try:
            if six.PY2:
                # Во втором питоне read() считает строку в байтовом виде,
                # и декодирует ее в unicode, с кодировкой из
                # settings.FILE_CHARSET
                template = open(filepath).read().decode(settings.FILE_CHARSET)
            else:
                # В третьем питоне можно указать кодировку файла до открытия,
                # метод read() вернет уже декодированную str
                template = open(filepath, encoding=settings.FILE_CHARSET).read()

                # В Django 2.2 settings.FILE_CHARSET вообще устарело,
                # все файлы, которые Django считывает с диска, сразу
                # должны быть в UTF-8.
                # При обновлении надо будет это все выкинуть
            return (template, filepath)
        except IOError:
            pass
    raise TemplateDoesNotExist(template_name)

load_template_source.is_usable = True

==> m3-ui-3.0.1.tar.gz: m3-ui-3.0.1/src/m3_ext/ui/js_template_loader.py <==
#coding: utf-8
"""
Загрузчик шаблонов (темплейтов) для генерации пользовательского интерфейса
для m3_ext_demo.ui.

Необходимость данного шаблона обусловлена спецификой реализации
template-loaders в django.

Для корректной работы загрузчика в settings.py прикладного приложения
необходимо добавить строку 'm3_ext_demo.ui.js_template_loader.load_template_source'
в tuple TEMPLATE_LOADERS
"""

import os
import sys

from django.conf import settings
from django.template import TemplateDoesNotExist
from django.utils._os import safe_join

# At compile time, cache the directories to search.
fs_encoding = sys.getfilesystemencoding() or sys.getdefaultencoding()
template_dir_ext = os.path.join(
    os.path.dirname(__file__), 'templates')
template_dir_gears = os.path.join(
    os.path.dirname(__file__), '../gears/templates')
app_template_dirs = (
    template_dir_ext.decode(fs_encoding),
    template_dir_gears.decode(fs_encoding),
)


def get_template_sources(template_name, template_dirs=None):
    """
    Returns the absolute paths to "template_name", when appended to each
    directory in "template_dirs". Any paths that don't lie inside one of the
    template dirs are excluded from the result set, for security reasons.
    """
    if not template_dirs:
        template_dirs = app_template_dirs
    for template_dir in template_dirs:
        try:
            yield safe_join(template_dir, template_name)
        except UnicodeDecodeError:
            # The template dir name was a bytestring that wasn't valid UTF-8.
            raise
        except ValueError:
            # The joined path was located outside of template_dir.
            pass


def load_template_source(template_name, template_dirs=None):
    for filepath in get_template_sources(template_name, template_dirs):
        try:
            return (
                open(filepath).read().decode(settings.FILE_CHARSET), filepath
            )
        except IOError:
            pass
    raise TemplateDoesNotExist(template_name)

load_template_source.is_usable = True
